package graph;

import java.util.Collections;
import java.util.TreeSet;
import java.util.Vector;

import main.Interface;

import utils.Point;
import utils.Polygon;
import utils.Segment;

public class VisibilityGraphCalculator {

    private static Point pivot;
    private static Segment directionOfSweepLine;
    private Vector<Point> vertexPointRelation;
    private Vector<Segment> segments;
    private Vector<Point> points;
    private Interface interfaceGrafica;

    public VisibilityGraphCalculator(Interface interfaceGrafica) {
        this.interfaceGrafica = interfaceGrafica;
    }

    public Graph calculate(Vector<Polygon> map, Point a, Point b) {
        
        
        initializeVectors();
        setSegmentsAndPointsVectors(map, a, b);
        Graph graph = new Graph(points.size() + 2);

        Vector<Point> orderedPointsByAngle = new Vector<Point>();
        TreeSet<Segment> sweepLine = new TreeSet<Segment>();

        //O(n² log n)
        for (int i = 0; i < points.size(); i++) {
            sweepLine.clear();
            orderedPointsByAngle.clear();
            interfaceGrafica.setSweep(null);
            interfaceGrafica.setSweepLineFirst(null);

            Point pivo = points.get(i);
            pivot = pivo;
            interfaceGrafica.setPivo(pivo);

            //O(n)
            for (int j = 0; j < points.size(); j++) {
                if (j != i) {
                    orderedPointsByAngle.add(points.get(j));
                }
            }

            //O(n log n)
            Collections.sort(orderedPointsByAngle);

            //O(n)
            Segment xAxis = new Segment(new Point(pivo.getX(), pivo.getY()), new Point(Double.MIN_VALUE, pivo.getY()));
            setDirection(xAxis);

            for (Segment segment : segments) {
                if (segment.intersect(xAxis) && !segment.getA().equals(pivo) && !segment.getB().equals(pivo)) {
                    sweepLine.add(segment);
                }
            }
            
            Vector<Segment> ss = new Vector<Segment>();
            for (Segment segment : sweepLine) {
                ss.add(segment);
            }
            interfaceGrafica.setSweep(ss);
            if (sweepLine.size() > 0) {
                interfaceGrafica.setSweepLineFirst(sweepLine.first());
            }
            if (interfaceGrafica.passo() == -1) {
                return null;
            }

            //O(n log n)
            for (int j = 0; j < orderedPointsByAngle.size(); j++) {

                Point point = orderedPointsByAngle.get(j);

                interfaceGrafica.setOtherPoint(point);


                //if(interfaceGrafica.passo() == -1)
                //	return null;


                Segment sweepLineFirst;

                if (sweepLine.size() > 0) {
                    sweepLineFirst = sweepLine.first();
                    interfaceGrafica.setSweepLineFirst(sweepLineFirst);
                } else {
                    sweepLineFirst = null;
                }

                //o point eh visivel pelo pivo
                boolean visivel = true;
                if (sweepLineFirst != null) {
                    if (!sweepLineFirst.intersect(new Segment(pivo, point))) {
                        if (pivo.getMyPolygon() == point.getMyPolygon()) {
                            if(Math.abs(pivo.getId()-point.getId()) != 1 && Math.abs(pivo.getId()-point.getId()) != map.get(pivo.getMyPolygon()).size()-1){
                                if (map.get(pivo.getMyPolygon()).in(new Point((pivo.getX() + point.getX()) / 2.0, (pivo.getY() + point.getY()) / 2.0))) {
                                    visivel = false;
                                }
                            }
                        }
                    }
                    else {
                        visivel = false;
                    }
                }

                if (visivel) {
                    double dist = pivo.dist(point);
                    graph.insert(pivo.getId(), point.getId(), dist);
                }

                interfaceGrafica.setGraph(graph);

                //Atualiza a sweep
                Segment seg1 = point.getRet1();
                Point otherPointofseg1 = null;
                if (seg1 != null) {
                    otherPointofseg1 = seg1.getB();
                }

                Segment seg2 = point.getRet2();
                Point otherPointofseg2 = null;
                if (seg2 != null) {
                    otherPointofseg2 = seg2.getA();
                }

                Segment sweepLineSegment = new Segment(pivo, point);
                Point ant = orderedPointsByAngle.get(j - 1 >= 0 ? j - 1 : orderedPointsByAngle.size() - 1);
                Point prox = orderedPointsByAngle.get((j + 1) % orderedPointsByAngle.size());
                Point medianAnt = new Point((ant.getX() + point.getX()) / 2.0, (ant.getY() + point.getY()) / 2.0);
                Point medianProx = new Point((prox.getX() + point.getX()) / 2.0, (prox.getY() + point.getY()) / 2.0);
                double mult = 1.0;
                while ((Math.abs((medianAnt.getX() - pivo.getX()) * mult) <= interfaceGrafica.getWidth()) ||
                       (Math.abs((medianAnt.getY() - pivo.getY()) * mult) <= interfaceGrafica.getHeight())) {
                    mult *= 2.0;
                }
                Segment dirToRemove = new Segment(pivo, new Point(pivo.getX() + (medianAnt.getX() - pivo.getX()) * mult,
                                                                  pivo.getY() + (medianAnt.getY() - pivo.getY()) * mult));

                mult = 1.0;
                while ((Math.abs((medianProx.getX() - pivo.getX()) * mult) <= interfaceGrafica.getWidth()) ||
                       (Math.abs((medianProx.getY() - pivo.getY()) * mult) <= interfaceGrafica.getHeight())) {
                    mult *= 2.0;
                }
                Segment dirToInsert = new Segment(pivo, new Point(pivo.getX() + (medianProx.getX() - pivo.getX()) * mult,
                                                                  pivo.getY() + (medianProx.getY() - pivo.getY()) * mult));

                setDirection(dirToRemove);
                interfaceGrafica.setDirection(dirToRemove);
                if (interfaceGrafica.passo() == -1) {
                    return null;
                }

                if (otherPointofseg1 != null && sweepLineSegment.left2(otherPointofseg1)==1 && !pivo.equals(otherPointofseg1)) {
                    sweepLine.remove(seg1);
                }
                if (otherPointofseg2 != null && sweepLineSegment.left2(otherPointofseg2)==1 && !pivo.equals(otherPointofseg2)) {
                    sweepLine.remove(seg2);
                }


                setDirection(dirToInsert);
                interfaceGrafica.setDirection(dirToInsert);
                if (interfaceGrafica.passo() == -1) {
                    return null;
                }

                if (otherPointofseg1 != null && sweepLineSegment.left2(otherPointofseg1)==-1 && !pivo.equals(otherPointofseg1)) {
                    sweepLine.add(seg1);
                }
                if (otherPointofseg2 != null && sweepLineSegment.left2(otherPointofseg2)==-1 && !pivo.equals(otherPointofseg2)) {
                    sweepLine.add(seg2);
                }


                ss = new Vector<Segment>();
                for (Segment segment : sweepLine) {
                    ss.add(segment);
                }
                interfaceGrafica.setSweep(ss);
                if (sweepLine.size() > 0) {
                    interfaceGrafica.setSweepLineFirst(sweepLine.first());
                }
            }
            interfaceGrafica.setGraph(graph);
        }
        Vector<Integer> parnt = graph.dijkstra(0, 1);

        Vector<Point> path = new Vector<Point>();
        int at = 1;
        while (parnt.get(at) >= 0) {
            path.add(vertexPointRelation.get(at));
            at = parnt.get(at);
        }
        path.add(vertexPointRelation.get(at));

        interfaceGrafica.setMinimumPath(path);

        //interfaceGrafica.setGraph(null);
        interfaceGrafica.setOtherPoint(null);
        interfaceGrafica.setPivo(null);
        interfaceGrafica.setSweep(null);
        interfaceGrafica.setSweepLineFirst(null);
        interfaceGrafica.setDirection(null);
        return graph;
    }

    private void initializeVectors() {
        vertexPointRelation = new Vector<Point>();
        segments = new Vector<Segment>();
        points = new Vector<Point>();
    }

    public Vector<Point> getVertexPointRelation() {
        return vertexPointRelation;
    }

    public static Point getPivot() {
        return pivot;
    }
    public static void setPivot(Point pivo) {
        pivot = pivo;
    }

    public static Segment getDirection() {
        return directionOfSweepLine;
    }

    public static void setDirection(Segment seg) {
        directionOfSweepLine = seg;
    }

    private void setSegmentsAndPointsVectors(Vector<Polygon> map, Point a, Point b) {
        int ids = 0;

        vertexPointRelation.clear();

        a.setId(ids++);
        a.setMyPolygon(-1);
        vertexPointRelation.add(a);
        points.add(a);
        b.setId(ids++);
        a.setMyPolygon(-2);
        vertexPointRelation.add(b);
        points.add(b);

        for (int i = 0; i < map.size(); i++) {
            for (int j = 0; j < map.get(i).size(); j++) {
                // Os id são o número do vértice referentes a esses pontos.

                Segment seg = new Segment(map.get(i).get(j), map.get(i).get((j + 1) % map.get(i).size()));

                map.get(i).get(j).setMyPolygon(i);
                map.get(i).get(j).setId(ids++);
                vertexPointRelation.add(map.get(i).get(j));

                map.get(i).get(j).setRet1(seg);
                map.get(i).get((j + 1) % map.get(i).size()).setRet2(seg);

                if (j != 0) {
                    points.add(map.get(i).get(j));
                }
                if (j == map.get(i).size() - 1) {
                    points.add(map.get(i).get(0));
                }

                segments.add(seg);
            }
        }
    }
}
