package utils;

import graph.VisibilityGraphCalculator;

public class Point implements Comparable<Point> {

    private double y;
    private double x;
    private int id;
    private Segment ret1;
    private Segment ret2;
    private int myPolygon;
    private final double EPSILON = 1e-9;

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Point) {
            Point ob = (Point) obj;
            if ((int) ob.getX() == (int) getX() && (int) ob.getY() == (int) getY()) {
                return true;
            }
        }
        return false;
    }

    public String toString() {
        return "(" + x + " , " + y + ")";
    }

    public Point(double a, double b) {
        setX(a);
        setY(b);
        setId(-1);
    }

    Point() {
        setX(0);
        setY(0);
    }

    int esq(Point a, Point b) {
        if (((a.getX() - getX()) * (b.getY() - getY())) - ((b.getX() - getX()) * (a.getY() - getY())) < 0) {
            return 1;
        }
        return 0;
    }

    public void setX(double x) {
        this.x = x;
    }

    public double getX() {
        return x;
    }

    public void setY(double y) {
        this.y = y;
    }

    public double getY() {
        return y;
    }

    public void setId(int id) {
        this.id = id;
    }

    public int getId() {
        return id;
    }

    public void setRet1(Segment ret1) {
        this.ret1 = ret1;
    }

    public Segment getRet1() {
        return ret1;
    }

    public void setRet2(Segment ret2) {
        this.ret2 = ret2;
    }

    public Segment getRet2() {
        return ret2;
    }
    
    @Override
    public int compareTo(Point arg0) {

        Point pivo = VisibilityGraphCalculator.getPivot();

        double atan2cmp = Math.atan2(arg0.getY() - pivo.getY(), arg0.getX() - pivo.getX());
        double atan2this = Math.atan2(getY() - pivo.getY(), getX() - pivo.getX());

        if (Math.abs(atan2cmp - atan2this) <= EPSILON) {
            return pivo.dist(this) < pivo.dist(arg0) ? -1 : 1;
        }
        return atan2cmp < atan2this ? -1 : 1;
    }

    public double dist2(Point arg0) {
        double xx = (arg0.getX() - getX()) * (arg0.getX() - getX());
        double yy = (arg0.getY() - getY()) * (arg0.getY() - getY());
        return xx + yy;
    }

    public double dist(Point arg0) {
        double xx = ((int)arg0.getX() - (int)getX()) * ((int)arg0.getX() - (int)getX());
        double yy = ((int)arg0.getY() - (int)getY()) * ((int)arg0.getY() - (int)getY());
        return Math.sqrt(xx + yy);
    }

    public void setMyPolygon(int i) {
        this.myPolygon = i;
    }

    public int getMyPolygon() {
        return myPolygon;
    }
}
