package utils;

import graph.VisibilityGraphCalculator;

public class Segment implements Comparable<Segment> {

    private static final double EPSILON = 1e-9;
    private Point a;
    private Point b;

    @Override
    public String toString() {
        return a.toString() + " , " + b.toString();
    }

    @Override
    public boolean equals(Object obj) {
        if(!(obj instanceof Segment))
            return false;
        Segment seg = (Segment) obj;
        int x1, x2, x3, x4, y1, y2, y3, y4;
        x1 = (int) a.getX();
        x2 = (int) b.getX();
        x3 = (int) seg.getA().getX();
        x4 = (int) seg.getB().getX();
        y1 = (int) a.getY();
        y2 = (int) b.getY();
        y3 = (int) seg.getA().getY();
        y4 = (int) seg.getB().getY();

        if ((Math.min(x1, x2) == Math.min(x3, x4)) && (Math.max(x1, x2) == Math.max(x3, x4)) && (Math.min(y1, y2) == Math.min(y3, y4)) && (Math.max(y1, y2) == Math.max(y3, y4))) {
            return true;
        }

        return false;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 19 * hash + (this.a != null ? this.a.hashCode() : 0);
        hash = 19 * hash + (this.b != null ? this.b.hashCode() : 0);
        return hash;
    }

    public Segment(Point a, Point b) {
        this.setA(a);
        this.setB(b);
    }

    public void setB(Point b) {
        this.b = b;
    }

    public Point getB() {
        return b;
    }

    public void setA(Point a) {
        this.a = a;
    }

    public Point getA() {
        return a;
    }

    public boolean intersect(Segment segment) {
        boolean A = segment.left(getA());
        boolean B = segment.left(getB());
        boolean C = this.left(segment.getA());
        boolean D = this.left(segment.getB());

        if (A != B && C != D) {
            return true;
        }
        return false;
    }

    public boolean left(Point point) {
        return (getA().getX() - point.getX()) * (getB().getY() - point.getY()) - (getA().getY() - point.getY()) * (getB().getX() - point.getX()) < -EPSILON;
    }

    public int left2(Point point) {
        if((getA().getX() - point.getX()) * (getB().getY() - point.getY()) - (getA().getY() - point.getY()) * (getB().getX() - point.getX()) < -EPSILON)
            return -1;
        else if((getA().getX() - point.getX()) * (getB().getY() - point.getY()) - (getA().getY() - point.getY()) * (getB().getX() - point.getX()) > EPSILON)
            return 1;
        return 0;
    }

    /*********************
     * Line intersection *
     *********************
     * Returns the point of intersection of two lines:
     * (x[0],y[0])-(x[1],y[1]) and (x[2],y[2])-(x[3],y[3]).
     * Puts the result (x, y) into (r[0], r[1]) and returns true.
     * If there is no intersection, return false;
     * USED BY: circle3pts
     * #include <math.h>
     * #define EPS ...
     **/
    public Point lineIntersect(Segment r) {
        double x0 = getA().getX();
        double y0 = getA().getY();
        double x1 = getB().getX();
        double y1 = getB().getY();
        double x2 = r.getA().getX();
        double y2 = r.getA().getY();
        double x3 = r.getB().getX();
        double y3 = r.getB().getY();
        double n0, n1, r0, r1;

        n0 = y3 - y2;
        n1 = x2 - x3;
        double denom = n0 * (x1 - x0) + n1 * (y1 - y0);
        if (Math.abs(denom) < EPSILON) {
            return null;
        }
        double num = n0 * (x0 - x2) + n1 * (y0 - y2);
        double t = -num / denom;
        r0 = x0 + t * (x1 - x0);
        r1 = y0 + t * (y1 - y0);
        return new Point(r0, r1);
    }

    @Override
    public int compareTo(Segment o) {

        if (this.equals(o)) {
            return 0;
        }
        Point p1 = lineIntersect(VisibilityGraphCalculator.getDirection());
        Point p2 = o.lineIntersect(VisibilityGraphCalculator.getDirection());
     
        return p1.dist2(VisibilityGraphCalculator.getPivot()) < p2.dist2(VisibilityGraphCalculator.getPivot()) ? -1 : 1;
    }
}
